// Color Fusion (Masked)
// tULAMIDE
// PS 1.4
// Layer effect for tinting with optional masking.

//#PARAM float chRed 64: Red : The red channel of the color.
float chRed;

//#PARAM float chGreen 32: Green : The green channel of the color.
float chGreen;

//#PARAM float chBlue 8: Blue : The blue channel of the color.
float chBlue;

//#PARAM percent cIntensity 0.7: Intensity : The strength of the color.
float cIntensity;

//#PARAM percent cMix 0.0: Mix Mask Color : The strength of the mask's color.
float cMix;

// Foreground texture
texture ForegroundTexture;

// Background texture
texture BackgroundTexture;

// Foreground sampler
sampler2D foreground = sampler_state {
    Texture = (ForegroundTexture);
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
};

// Background sampler
sampler2D background = sampler_state {
    Texture = (BackgroundTexture);
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
};

//Tinting 
float3 ctint(float3 ocolor, float3 tintcolor, float strength)
{	
	float grey = (ocolor.r + ocolor.g + ocolor.b) / 3;
	
	float nRed = (1.0f - strength) * ocolor.r + (strength * tintcolor.r * grey);
	float nGrn = (1.0f - strength) * ocolor.g + (strength * tintcolor.g * grey);
	float nBlu = (1.0f - strength) * ocolor.b + (strength * tintcolor.b * grey);
        
	return float3(nRed, nGrn, nBlu);
}

// Effect function
float4 EffectProcess( float2 Tex : TEXCOORD0 ) : COLOR0
{
	float4 colorF = tex2D(foreground, Tex.xy);
	float4 colorB = tex2D(background, Tex.xy);
	float4 color;
	
	if(colorF.a == 0)
	{
		color.rgb = ctint(colorB.rgb, float3(chRed/255, chGreen/255, chBlue/255), cIntensity);
	}
	else
	{
		color.rgb = ctint(colorB.rgb, float3(chRed/255, chGreen/255, chBlue/255), cIntensity * (1.0f - colorF.a));
		if(cMix != 0)
		{
			colorF.rgb /= colorF.a;
			color.rgb = lerp(color.rgb, colorF.rgb, colorF.a * cMix * cIntensity);
		}
	}
	
	color.a = colorB.a;
	color.rgb *= color.a;
	return color;
}
	
// ConstructEffect
technique ConstructEffect
{
    pass p0
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 EffectProcess();
    }
}
