// Curves
// Paulo Ricardo Reinehr
// PS 2.0
// Abstract implementation of tonal curves for RGB, Red, Green and Blue channels with three fixed nodes: highlights, midtones and shadows. Can be used to adjust overall contrast or create advanced color filters.


//#PARAM percent highlights_rgb 0 : RGB Highlights : Positive values brighten overall highlights, negative values darken overall highlights.
float highlights_rgb;

//#PARAM percent midtones_rgb 0 : RGB Midtones : Positive values brighten overall midtones, negative values darken overall midtones.
float midtones_rgb;

//#PARAM percent shadows_rgb 0 : RGB Shadows : Positive values brighten overall shadows, negative values darken overall shadows.
float shadows_rgb;

//#PARAM percent highlights_r 0 : R Highlights : Positive values tint highlights red, negative values tint highlights cyan.
float highlights_r;

//#PARAM percent midtones_r 0 : R Midtones : Positive values tint midtones red, negative values tint midtones cyan.
float midtones_r;

//#PARAM percent shadows_r 0 : R Shadows : Positive values tint shadows red, negative values tint shadows cyan.
float shadows_r;

//#PARAM percent highlights_g 0 : G Highlights : Positive values tint highlights green, negative values tint highlights magenta.
float highlights_g;

//#PARAM percent midtones_g 0 : G Midtones : Positive values tint midtones green, negative values tint midtones magenta.
float midtones_g;

//#PARAM percent shadows_g 0 : G Shadows : Positive values tint shadows green, negative values tint shadows magenta.
float shadows_g;

//#PARAM percent highlights_b 0 : B Highlights : Positive values tint highlights blue, negative values tint highlights yellow.
float highlights_b;

//#PARAM percent midtones_b 0 : B Midtones : Positive values tint midtones blue, negative values tint midtones yellow.
float midtones_b;

//#PARAM percent shadows_b 0 : B Shadows : Positive values tint shadows blue, negative values tint shadows yellow.
float shadows_b;


// Foreground texture
texture ForegroundTexture;

// Background texture
texture BackgroundTexture;


// Foreground sampler
sampler2D foreground = sampler_state {
    Texture = (ForegroundTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};

// OriginalTexture sampler
sampler2D background = sampler_state {
    Texture = (BackgroundTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};


// Effect function
float4 EffectProcess( float2 Tex : TEXCOORD0 ) : COLOR0
{

	float4 color = tex2D(foreground, Tex.xy);

	float3 highlights = highlights_rgb * 0.5 + float3( highlights_r , highlights_g , highlights_b ) * 0.5  ;
	float3 midtones = midtones_rgb * 0.5 + float3( midtones_r , midtones_g , midtones_b ) * 0.5 ;
	float3 shadows = shadows_rgb * 0.5 + float3( shadows_r , shadows_g , shadows_b ) * 0.5 ;

	float3 x = lerp( midtones , highlights , color.rgb ) ;
	float3 y = lerp( shadows , midtones , color.rgb ) ;

	color.rgb += lerp( y , x , color.rgb );

	return color ;

}

// ConstructEffect
technique ConstructEffect
{
    pass p0
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 EffectProcess();
    }
}
